// :  

#include <QtCore>
#include "QtWebSockets/qwebsocketserver.h"
#include "QtWebSockets/qwebsocket.h"
#include "QtServer.h"

int clientCount = 0;

Server::Server(quint16 port, bool debug, QObject *parent) :
	QObject(parent),
	m_pWebSocketServer(new QWebSocketServer(QStringLiteral("Server"),
		QWebSocketServer::NonSecureMode, this)),
	m_debug(debug)
{
    if (m_pWebSocketServer->listen(QHostAddress::Any, port)) {
		if (m_debug)
            qDebug() << "Listening on port" << port;
		connect(m_pWebSocketServer, &QWebSocketServer::newConnection,
			this, &Server::onNewConnection);
		connect(m_pWebSocketServer, &QWebSocketServer::closed, this, &Server::closed);
	}
    else
        qDebug() << "Error: can't listen. Check port is free?";
}

Server::~Server()
{
	m_pWebSocketServer->close();
	qDeleteAll(m_clients.begin(), m_clients.end());
}

void Server::onNewConnection()
{
	QWebSocket *pSocket = m_pWebSocketServer->nextPendingConnection();
	++clientCount;
	qDebug() << clientCount << " clients connected";
	
	connect(pSocket, &QWebSocket::textMessageReceived, this, &Server::processTextMessage);
	connect(pSocket, &QWebSocket::disconnected, this, &Server::socketDisconnected);

	m_clients << pSocket;
}

void Server::processTextMessage(QString message)
{
	QWebSocket *pClient = qobject_cast<QWebSocket *>(sender());

	foreach(auto x, message.split("\n")) {
		try {
			QStringList arg = x.split(" ");

			if (arg.size() != 4 || arg[0] != "CALC") {
				pClient->sendTextMessage("ERR");
				continue;
			}

			bool ok;

			int ch1 = arg[2].toInt(&ok);
			int ch2 = arg[3].toInt(&ok);

			if (!ok) {
				pClient->sendTextMessage("ERR");
				continue;
			}

			if (arg[1] == "*") {
				int result = ch1 * ch2;
				pClient->sendTextMessage(QString::number(result));
				continue;
			}

			if (arg[1] == "+") {
				int result = ch1 + ch2;
				pClient->sendTextMessage(QString::number(result));
				continue;
			}

			if (arg[1] == "-") {
				int result = ch1 - ch2;
				pClient->sendTextMessage(QString::number(result));
				continue;
			}

			if (arg[1] == "/") {
				int result = ch1 / ch2;
				pClient->sendTextMessage(QString::number(result));
				continue;
			}
			else
			{
				pClient->sendTextMessage("ERR");
				continue;
			}
		}
		catch (...) {
			pClient->sendTextMessage("ERR");
			continue;
		}
	}
}

void Server::socketDisconnected()
{
	QWebSocket *pClient = qobject_cast<QWebSocket *>(sender());
	--clientCount;
	qDebug() << clientCount << " clients connected";
	
	if (pClient) {
		m_clients.removeAll(pClient);
		pClient->deleteLater();
	}
}


